TeamEquipCompare = LibStub("AceAddon-3.0"):NewAddon("TeamEquipCompare", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0");
local lib = TeamEquipCompare;
lib.version = "0.4";
lib.commPrefix = "TEC_COMMS";
lib.player = {};
lib.party = {};

local defaultDB = {
	global = {
		version = lib.version,
		showDebugMsgs = false,
		showNotificationMsgs = true,
		requireCtrlKey = true,
		trackMoney = true,
		moneyThreshold = 5,
	}
};

function lib:notify(msg)
	if (lib.db.global.showNotificationMsgs) then self:Print(msg) end
end

function lib:debug(msg)
	if (lib.db.global.showDebugMsgs) then self:Print("[DEBUG]: "..msg) end
end

function lib:OnInitialize()
	-- load global saved variable, and update with new values if needed
	lib.db = LibStub("AceDB-3.0"):New("TEC_DB", defaultDB, true);
	if (lib.db.version ~= defaultDB.version) then
		lib:notify("Version difference found, setting new values to default...");
		lib.db = TEC_UpdateSavedDB(lib.db, defaultDB);
	end

	-- load player dataset one time data values
	lib.player.name = GetUnitName("player", false);
	lib.player.class = UnitClass("player");

	-- set initial money value
	lib.lastMoneyUpdate = GetMoney() or 0;

	-- register communication channel
	self:RegisterComm(lib.commPrefix, "OnCommReceived");

	-- register events
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "OnGroupRosterUpdate");
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "OnPlayerEquipmentChanged");
	self:RegisterEvent("PLAYER_LEVEL_UP", "OnPlayerLevelUp");
	self:RegisterEvent("PLAYER_MONEY", "OnPlayerMoney");
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "OnPlayerSpecializationChanged");

	-- register command options, and add tab to Blizzards Interface/AddOns GUI
	LibStub("AceConfig-3.0"):RegisterOptionsTable("TeamEquipCompare", self.options, "tec");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("TeamEquipCompare", "Team Equip Compare");	

	self:Print(format("TeamEquipCompare v%s loaded", lib.version));
end

-- TODO: do I need to handle enabling/disabling addon?
function lib:OnEnable()
	self:Print(format("TeamEquipCompare v%s enabled", lib.version));
end

function lib:OnDisable()
	self:Print(format("TeamEquipCompare v%s disabled", lib.version));
end

function lib:OnGroupRosterUpdate(...)
	self:debug("OnGroupRosterUpdate Fired");

	-- clear data if not grouped
	if (not IsInGroup()) then
		self:debug("Clearing party data...");
		lib.party = {};

	-- otherwise, update the data as needed
	else
		local unitsToAdd, unitsToRemove = TEC_CheckTeamMembers();

		-- add any new party members
		if (#unitsToAdd > 0) then
			for ii=1, #unitsToAdd do
				-- add key to party table to prevent multiple requests
				lib.party[unitsToAdd[ii]] = { player = { name = unitsToAdd[ii] } };
				TEC_RequestUnitData(unitsToAdd[ii]);
			end
		end

		-- remove orphaned party members
		if (#unitsToRemove > 0) then
			for ii=1, #unitsToRemove do
				self:notify(format("Removing data for '%s'...", unitsToRemove[ii]));
				lib.party[unitsToRemove[ii]] = nil;
			end
		end
	end
end

function lib:OnPlayerEquipmentChanged(event, slotid, ...)
	self:debug(format("PlayerEquipmentChanged Event Fired, slotid: %d", slotid));

	-- report my data to team if my equipment changed
	if (IsInGroup()) then
		TEC_SendPlayerData("PARTY");
	end
end

function lib:OnPlayerLevelUp(event, level, ...)
	self:debug(format("OnPlayerLevelUp Event Fired: new level (%d): ", level));

	-- report my data to team when my level changes
	if (IsInGroup() and (level)) then
		TEC_SendLevelUpdate(level);
	end
end

function lib:OnPlayerMoney(...)
	self:debug("PlayerMoney Event Fired");
	
	if (lib.db.global.trackMoney) and (IsInGroup()) then
		-- get current player money
		local money = GetMoney();

		-- test to see if the amount has crossed the threshold (in gold), up or down
		if (abs(money - lib.lastMoneyUpdate) > (lib.db.global.moneyThreshold * 10000)) then
			-- send update to team
			TEC_SendMoneyUpdate(money);

			-- update static value
			lib.lastMoneyUpdate = money;
		end
	end
end

function lib:OnPlayerSpecializationChanged(event, unit, ...)
	self:debug(format("OnPlayerSpecializationChanged Event Fired: %s, %s", tostring(event), tostring(unit)));

	-- report my spec to the team if it changes
	if (IsInGroup() and unit == 'player') then
		TEC_SendSpecUpdate(TEC_GetSpecName());
	end
end

function lib:OnCommReceived(_, commMsg, _, unit, ...)
	self:debug(format("OnCommReceived -- %s: %s", unit, tostring(commMsg)));

	-- request to send player data to the party
	if (commMsg == "SYNC") then
		TEC_SendPlayerData("PARTY");

	-- request to send player data to the party
	elseif (commMsg == "REQUEST") then
		TEC_SendPlayerData(unit);

	-- receive level update
	elseif (strfind(commMsg, "LEVEL")) then
		local level = strmatch(commMsg, "LEVEL=(.*)");
		self:notify(format("Received level update from '%s' (%s)", unit, level));
		lib.party[unit].level = tonumber(level);

	-- receive money update
	elseif (strfind(commMsg, "MONEY")) then
		local money = strmatch(commMsg, "MONEY=(.*)");
		self:debug(format("Received money update from '%s' (%s)", unit, money));
		lib.party[unit].money = tonumber(money);

	-- receive spec update
	elseif (strfind(commMsg, "SPEC")) then
		local spec = strmatch(commMsg, "SPEC=(.*)");
		self:debug(format("Received spec update from '%s' (%s)", unit, spec));
		lib.party[unit].spec = spec;

	-- otherwise, receive team member data
	else
		local data = TEC_UnpackageData(commMsg);
		if (data) and (data ~= "") then
			lib.party[unit] = data;
			self:notify(format("Received update from '%s'", unit));
		else
			self:notify(format("Error receiving data from %s", unit));
		end
	end
end